#!/usr/bin/env node
"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
}
Object.defineProperty(exports, "__esModule", { value: true });
const commander_1 = __importDefault(require("commander"));
const xray_trace_fetcher_1 = require("../lib/xray-trace-fetcher");
const logger_1 = require("../lib/logger");
const lodash_1 = require("lodash");
const index_1 = require("../index");
// tslint:disable-next-line:no-var-requires
const version = require('./../../package.json').version;
let parseError = false;
function error(msg, ...args) {
    console.error(`\nError: ${msg}`, args);
}
/**
 * Print help and exit
 */
function helpError() {
    commander_1.default.help();
    process.exit(1);
}
/**
 * Will parse an int but if fails will print an error and set parseError to true.
 */
function parseIntWithError(s, def) {
    const i = parseInt(s, 10);
    if (isNaN(i)) {
        parseError = true;
        error("Failed parsing integer value: [%s]", s);
    }
    return i;
}
// tslint:disable:max-line-length
commander_1.default.version(version)
    .option('-s, --start-time <timestamp>', 'Start time as Unix timestamp (seconds since 1970-01-01 00:00:00 UTC). Optional: if left out will use: (current time - time range).', parseIntWithError)
    .option('-r, --time-range <minutes>', 'Time range in minutes to scan from start time.', parseIntWithError, 60)
    .option('-c, --compare', 'Compare current role and generated roles. Output a json report.')
    .option('-v, --verbose', 'Output verbose logs to the console (info and above).')
    .option('-f, --filter <expression>', 'Filter expression to use when scanning xray. See AWS docs for synatx.')
    .parse(process.argv);
// tslint:enable:max-line-length
if (parseError) {
    helpError();
}
//sanity check on start time
const now = Date.now();
const startTime = commander_1.default.startTime ? commander_1.default.startTime * 1000 : undefined;
if (startTime && (startTime < (now - 30 * 24 * 60 * 60 * 1000) || startTime > (now + 60 * 1000))) {
    error("Invalid start time [%s]. Start time can't be older than 30 days or in the future.", commander_1.default.startTime);
    helpError();
}
const conf = {
    startTime: startTime ? new Date(startTime) : undefined,
    timeRangeMinutes: commander_1.default.timeRange,
    filterExpression: commander_1.default.filter,
};
if (commander_1.default.verbose) {
    logger_1.changeConsoleLevel('info');
    logger_1.logger.info("Console logging set to verbose level: [info]");
}
if (commander_1.default.compare) {
    logger_1.logger.info("Enabling role comparison.");
    conf.compareExistingRole = true;
}
xray_trace_fetcher_1.scanXrayAndSaveFiles(conf)
    .then((res) => {
    console.log('Completed running xray scan.');
    if (!lodash_1.isEmpty(res.GeneratedPolicies)) {
        console.log("Generated IAM policies based upon xray scan:");
        for (const p of res.GeneratedPolicies) {
            console.log(`${p.Arn} - ${p.FileName}`);
        }
    }
    else {
        console.log("No IAM policies generated");
    }
    if (!lodash_1.isEmpty(res.ExcessPermissions)) {
        console.log("Found excessive permissions. Result written out to: ", index_1.EXCESSIVE_PERMISSION_FILE);
    }
    else if (commander_1.default.compare) {
        console.log("No excessive permisssions found.");
    }
})
    .catch((err) => {
    console.error('Unexpected Error: Failed running scan: ', err);
});
//# sourceMappingURL=xray-privilege-scan.js.map