"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
}
Object.defineProperty(exports, "__esModule", { value: true });
const logger_1 = require("./logger");
const aws_sdk_1 = __importDefault(require("aws-sdk"));
const lodash_1 = __importDefault(require("lodash"));
const index_1 = require("./aws-segments/index");
const bluebird_1 = __importDefault(require("bluebird"));
const crypto_1 = __importDefault(require("crypto"));
const fs_1 = __importDefault(require("fs"));
const iam_policy_utils_1 = require("./iam-policy-utils");
const DEF_TIME_RANGE = 60;
exports.EXCESSIVE_PERMISSION_FILE = 'excessive_permissions.json';
/**
 * Fetch xray traces. Combines the two functions of getTraceSummaries and batchGetTraces including support for pagination.
 */
function getXrayTraces(conf) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            logger_1.logger.info('getXrayTraces: params: ', conf);
            const xray = new aws_sdk_1.default.XRay();
            const timeRangeMS = ((conf && conf.timeRangeMinutes) || DEF_TIME_RANGE) * 60 * 1000;
            const StartTime = (conf && conf.startTime) || new Date(Date.now() - timeRangeMS);
            const EndTime = new Date(StartTime.getTime() + timeRangeMS);
            logger_1.logger.info("GetTraceSummariesRequest: StarTime: %s, EndTime: %s", StartTime, EndTime);
            const summariesParams = {
                StartTime,
                EndTime,
                FilterExpression: (conf && conf.filterExpression) || undefined,
            };
            let paginationNum = 0;
            let nextToken;
            let summaries = [];
            do {
                summariesParams.NextToken = nextToken;
                const traceSummariesResp = yield xray.getTraceSummaries(summariesParams).promise();
                logger_1.logger.debug('getTraceSummaries [token: %s] result: ', nextToken, traceSummariesResp);
                if (traceSummariesResp.TraceSummaries) {
                    summaries = summaries.concat(traceSummariesResp.TraceSummaries);
                }
                nextToken = traceSummariesResp.NextToken;
                paginationNum++;
            } while (nextToken);
            logger_1.logger.info('getTraceSummaries total summaries after [%s] pagination: ', paginationNum, summaries.length);
            let batchNum = 0;
            let ids = [];
            // tslint:disable-next-line:max-line-length
            const batchResults = yield bluebird_1.default.map(summaries, (trace, index, arrayLength) => {
                if (trace.Id) {
                    ids.push(trace.Id);
                }
                //aws enforces max 5 ids in a batch. We do the request when we reach 5 or on the last index
                if (ids.length === 5 || index === (arrayLength - 1)) {
                    logger_1.logger.debug('Performing batchGetTraces with ids: ', ids);
                    const res = xray.batchGetTraces({ TraceIds: ids }).promise();
                    ids = [];
                    batchNum++;
                    return res;
                }
                return null;
            }, { concurrency: 10 });
            let traces = [];
            for (const res of batchResults) {
                if (res) {
                    logger_1.logger.debug("batchGetTraces result: ", res);
                    if (!lodash_1.default.isEmpty(res.Traces)) {
                        logger_1.logger.debug('batchGetTraces count: ', res.Traces.length);
                        traces = traces.concat(res.Traces);
                    }
                    else {
                        logger_1.logger.error('Got empty result from batchGetTraces for ids: ', ids);
                    }
                }
            }
            logger_1.logger.info('getXrayTraces total traces returned: %s, number of batches: %s', traces.length, batchNum);
            return traces;
        }
        catch (error) {
            logger_1.logger.error("Failed fetching traces: ", error);
            throw error;
        }
    });
}
exports.getXrayTraces = getXrayTraces;
function parseSegmentDoc(doc, functionMap, parentActionsMap, functionArn) {
    //if this is a subsegment we will want to add to the parentActionsMap if this relates to an aws service
    if (parentActionsMap && doc.namespace === 'aws' && index_1.SUPPORTED_SERVICES.indexOf(doc.name) >= 0) {
        index_1.AWSSegmentParsers[doc.name](doc, parentActionsMap, functionArn);
    }
    //see if this is a Lambda function. If so it will have aws.function_arn
    const arn = lodash_1.default.get(doc, 'aws.function_arn');
    let actionsMap;
    if (arn) {
        logger_1.logger.debug('Found arn: %s for Segment: %s', arn, doc.id);
        actionsMap = functionMap.get(arn);
        if (!actionsMap) {
            actionsMap = new Map();
            functionMap.set(arn, actionsMap);
        }
    }
    if (!lodash_1.default.isEmpty(doc.subsegments)) {
        for (const subsegment of doc.subsegments) {
            parseSegmentDoc(subsegment, functionMap, actionsMap, arn);
        }
    }
}
/**
 * Parse an xray trace return the passed in functionMap
 * @param trace
 * @param functionMap will populate with the found results
 */
function parseXrayTrace(trace, functionMap) {
    if (lodash_1.default.isEmpty(trace.Segments)) {
        logger_1.logger.info('No segments found for TraceId: ', trace.Id);
        return functionMap;
    }
    //we search for lambda segments and then follow into sub segments
    for (const segment of trace.Segments) {
        const docStr = segment.Document;
        if (lodash_1.default.isEmpty(docStr)) {
            logger_1.logger.warn("Got segment [id: %s] with empty Document.", segment.Id);
        }
        else {
            parseSegmentDoc(JSON.parse(docStr), functionMap);
        }
    }
    return functionMap;
}
exports.parseXrayTrace = parseXrayTrace;
function parseXrayTraces(traces, functionMap) {
    if (lodash_1.default.isEmpty(traces)) {
        return functionMap;
    }
    for (const trace of traces) {
        parseXrayTrace(trace, functionMap);
    }
    return functionMap;
}
exports.parseXrayTraces = parseXrayTraces;
function getFunctionActionMapFromXray(conf) {
    return __awaiter(this, void 0, void 0, function* () {
        const traces = yield getXrayTraces(conf);
        const map = new Map();
        return parseXrayTraces(traces, map);
    });
}
exports.getFunctionActionMapFromXray = getFunctionActionMapFromXray;
function createIAMPolicyDoc(map, functionArn) {
    const doc = {
        Version: "2012-10-17",
        Description: "Generated policy from xray scan for: " + functionArn,
        Statement: [],
    };
    //consolidate resources with the same functions into one entry
    const actionMap = new Map();
    for (const entry of map.entries()) {
        const match = entry[0].match(/arn:aws:(.+?):/);
        let service = '';
        if (!match || match.length < 2) {
            logger_1.logger.error("Couldn't extract service name from resource arn: ", entry[0]);
        }
        else {
            service = match[1];
        }
        const actions = Array.from(entry[1], (s) => `${service}:${s}`).sort();
        const actionsKey = actions.join();
        let actionMapEntry = actionMap.get(actionsKey);
        if (!actionMapEntry) {
            actionMapEntry = {
                resources: [],
                actions,
            };
            actionMap.set(actionsKey, actionMapEntry);
        }
        actionMapEntry.resources.push(entry[0]);
    }
    for (const val of actionMap.values()) {
        const stm = {
            Effect: "Allow",
            Action: val.actions,
            Resource: val.resources,
        };
        doc.Statement.push(stm);
    }
    return doc;
}
exports.createIAMPolicyDoc = createIAMPolicyDoc;
function compareLambdaRoleToPolicy(lambdaArn, policy) {
    return __awaiter(this, void 0, void 0, function* () {
        const iamUtils = new iam_policy_utils_1.IamPolicyUtils();
        if (!policy.Statement) {
            return;
        }
        const lambda = new aws_sdk_1.default.Lambda();
        const config = yield lambda.getFunctionConfiguration({ FunctionName: lambdaArn }).promise();
        if (config.Role) {
            const roleName = config.Role.substr(config.Role.lastIndexOf('/') + 1);
            const roleStatements = yield iamUtils.getCombinedStatementsForRole(roleName);
            const excessStatements = iamUtils.checkExcessPolicyPermissions(roleStatements, policy.Statement);
            if (!lodash_1.default.isEmpty(excessStatements)) {
                const res = {
                    arn: lambdaArn,
                    role: config.Role,
                    excessPermissions: excessStatements,
                };
                return res;
            }
        }
    });
}
exports.compareLambdaRoleToPolicy = compareLambdaRoleToPolicy;
/**
 * Will scan xray and return a ScanResult
 * @param conf
 */
function scanXray(conf) {
    return __awaiter(this, void 0, void 0, function* () {
        const map = yield getFunctionActionMapFromXray(conf);
        const res = {
            GeneratedPolicies: [],
            ExcessPermissions: [],
        };
        const promiseArr = [];
        for (const entry of map.entries()) {
            const policyDoc = createIAMPolicyDoc(entry[1], entry[0]);
            res.GeneratedPolicies.push({
                Arn: entry[0],
                Policy: policyDoc,
            });
            if (conf.compareExistingRole) {
                promiseArr.push(compareLambdaRoleToPolicy(entry[0], policyDoc));
            }
        }
        if (conf.compareExistingRole) {
            const permissionsRes = yield Promise.all(promiseArr);
            for (const perm of permissionsRes) {
                if (perm) {
                    res.ExcessPermissions.push(perm);
                }
            }
        }
        return res;
    });
}
exports.scanXray = scanXray;
/**
 *
 * @param conf
 * @return object mapping function arn to file name
 */
function scanXrayAndSaveFiles(conf) {
    return __awaiter(this, void 0, void 0, function* () {
        const res = yield scanXray(conf);
        for (const generatedPolicy of res.GeneratedPolicies) {
            const hash = crypto_1.default.createHash('md5');
            hash.update(generatedPolicy.Arn);
            const fileName = hash.digest('hex') + ".policy.json";
            generatedPolicy.FileName = fileName;
            try {
                fs_1.default.writeFileSync(fileName, JSON.stringify(generatedPolicy.Policy, undefined, 2));
            }
            catch (error) {
                logger_1.logger.error("Failed writing to file: [%s] ", fileName, error);
            }
        }
        if (!lodash_1.default.isEmpty(res.ExcessPermissions)) {
            fs_1.default.writeFileSync(exports.EXCESSIVE_PERMISSION_FILE, JSON.stringify(res.ExcessPermissions, undefined, 2));
        }
        return res;
    });
}
exports.scanXrayAndSaveFiles = scanXrayAndSaveFiles;
// export XRay.Trace;
//# sourceMappingURL=xray-trace-fetcher.js.map